import os

import jinja2 as jj
from systemrdl.node import RegNode
from . import TEMPLATE_DIR
from .loader import SimpleLoader


# Tests
def is_filled_by_field(node: RegNode) -> bool:
    """Jinja test to decide if a register is filled by a single field.

    Args:
        node (RegNode): Node to be checked.

    Returns:
        bool: returns whether the node is filled entirly by a single field.
    """
    return len(node.inst.children) == 1 and node.inst.children[0].width == get_field_width(node)


def is_write_only(node: RegNode) -> bool:
    """Jinja test to decide if a register is software write only.

    Args:
        node (RegNode): Node to be checked.

    Returns:
        bool: returns whether the register is write only.
    """
    return node.has_sw_writable and not node.has_sw_readable


def is_read_only(node: RegNode) -> bool:
    """Jinja test to decide if a register is software read only.

    Args:
        node (RegNode): Node to be checked.

    Returns:
        bool: returns whether the register is read only.
    """
    return node.has_sw_readable and not node.has_sw_writable


# Filters
def get_field_width(node: RegNode) -> int:
    """Filter to get the width of a Reg.

    Args:
        node (RegNode): Reg for which to return the width.

    Returns:
        int: width of register in bits.
    """
    return node.size * 8


def register_helpers(jj_env: jj.Environment) -> None:
    """Prepares the jinja environment by registering all tests and filters to be available inside
    the templates.

    Args:
        jj_env (jj.Environment): The Environment for which to register the tests and filters.
    """
    jj_env.tests["single_field"] = is_filled_by_field
    jj_env.tests["readonly"] = is_read_only
    jj_env.tests["writeonly"] = is_write_only

    jj_env.filters["width"] = get_field_width


class OutputFileGenerator():
    """Generator that fills the template.
    """

    def __init__(self) -> None:
        loader = SimpleLoader([TEMPLATE_DIR, os.getcwd()])
        jj_env = jj.Environment(loader=loader)
        register_helpers(jj_env)
        self.jj_env = jj_env

    def run(self,
            template_name: str,
            context: dict,
            output_filename: str,
            overwrite: bool = False,
            never_overwrite: bool = False) -> None:
        """Fetches and fills the requested template with the given name.

        Only generates a new file if the desired file (content) does not already exist. 
        This behavior can be overridden by using the overwrite option.

        Also offers never_overwrite, which will not allow any new file creation if a file with
        the desired output name already exists.

        Args:
            template_name (str): Name of the template in the 'templates' directory or 
            (relative) path to custom template.
            context (dict): Available context inside the template.
            output_filename (str): Name of the file that will be created.
            overwrite (bool): Force overwrite if files already exist.
            never_overwrite (bool): Protection against overwrites.
        """
        if os.path.isfile(output_filename):
            if never_overwrite:
                return
            if not overwrite:
                gen_content = self.jj_env.get_template(
                    template_name).render(context)
                with open(output_filename, 'r', encoding='utf8') as existing_file:
                    existing_content = existing_file.read()
                    if gen_content == existing_content:
                        return
        with open(output_filename, 'w', encoding='utf8') as f:
            self.jj_env.get_template(template_name).stream(context).dump(f)
