
import sys
import argparse
import tempfile

from pathlib import Path
from systemrdl import RDLCompiler
from systemrdl import component as comp
from peakrdl.main import main as peakrdl_main


def parse_args():
    parser = argparse.ArgumentParser(
        description="""
        Wrapper for the peakrdl_mnrs exporter. When supplied with a rdl containing an addrmap it forwards it to the peakrdl_mnrs exporter,
        creating the corresponding header file.
        If a rdl containing a regfile is supplied it wraps it into a dummy addrmap to make it legal for peakrdl and then causes the exporter to
        generate the correct regfile header file(s).
        """)
    # Required arguments
    parser.add_argument('files', metavar='FILE', help='Input file')
    parser.add_argument('-o', '--output', help='Output path', required=True)

    # Optional arguments
    parser.add_argument('-I', dest='incdir', metavar='INCDIR', action='append',
                        help='Search directory for files included with `include "filename"')
    parser.add_argument('--addrmap', action='store_true',
                        help='Generate addressmap header file.')
    parser.add_argument('--regfiles', action='store_true',
                        help='Generate regfile headers. If no addrmap is present it wraps the regfiles into a dummy.')
    parser.add_argument(
        '--template', help='Set custom template.')
    parser.add_argument('--overwrite', action='store_true',
                        help='Force generation of files, independent of whether they already exist or not.')

    return parser.parse_args()


def main():
    args = parse_args()
    rdlc = RDLCompiler()
    incdir = args.incdir if args.incdir else []
    rdlc.compile_file(args.files, incdir)
    addrmap_present = any(isinstance(comp_def, comp.Addrmap)
                          for comp_def in rdlc.root.comp_defs.values())
    if not addrmap_present:
        new_rdlc = RDLCompiler()
        files_to_include = {
            component.def_src_ref.filename for component in rdlc.root.comp_defs.values()}
        with tempfile.NamedTemporaryFile('w+', suffix=".rdl") as f:
            for include in files_to_include:
                f.write(f'`include "{Path(include).absolute()}"\n')
            f.write("addrmap wrapper_dummy_map{\n")
            for regfile_def in rdlc.root.comp_defs.values():
                if isinstance(regfile_def, comp.Regfile):
                    type_name = regfile_def.type_name
                    f.write(f"{type_name} {type_name}_instance;\n")
            f.write("};")
            f.seek(0)
            new_rdlc.compile_file(f.name, incdir)
            rdlc = new_rdlc
            sys.argv = ["", "mnrs", "--regfiles"]+sys.argv[1:]
            to_change = sys.argv.index(args.files)
            sys.argv[to_change] = f.name
            peakrdl_main()
    else:
        sys.argv = ["", "mnrs", "--addrmap"]+sys.argv[1:]
        peakrdl_main()


if __name__ == "__main__":
    main()
